/*
Copyright 2004 Ross Litscher

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <iostream>
#include <fstream>
#include <string>  

#include "df3.hpp"
#include "cl_args.hpp"

// exception headers
#include "df3_e.hpp"
#include "tga_e.hpp"
#include "cl_args_e.hpp"
#include "DF3Exceptions.hpp"

int main (int argc, char* argv[])
{
    // TODO: try to move args and df3file to objects, instead of pointers to objects
    arg::CL_Args* args = NULL; // set to NULL for a test in catch(...)

    int major = 0; // set version numbers assuming there
    int minor = 8; // are "releases" of this upon the wild

    try {
	arg::CL_Args* args = new arg::CL_Args(argc, argv);
	args->readCL();

	if (args->Version()){
	    // print version and exit
	    args->printVersion(major, minor);
	    return 0;
	}

	if (args->Help()) {
	    // print help and exit
	    args->printHelp();
	    return 0;
	}

	std::string infiles;
	std::vector<std::string> file_q;

	if (args->usingListfile()) {
	    // if a list of files was specified, read each line
	    // and push into a queue of strings
	    std::ifstream input(args->Listfile().c_str());
	    
	    if (input.is_open()) {
		while (getline(input, infiles)){
		    file_q.push_back(infiles);
		}
	    } else {
		throw Exceptions::IOError(args->Listfile());
	    }
	} else {
	    // here, neither -b nor -f were specified so try reading from standard input. 
	    // this will seem to hang if no input is given. 
	    while (getline(std::cin, infiles)){
		file_q.push_back(infiles);
	    }
	}
	
	// allow for sorting the list of files before processing
	if (args->Sort()){ sort(file_q.begin(), file_q.end()); }
	std::string df3name;
	if (args->setOutput()) {
	    // set the name of df3 output file from the -o command line value
	    df3name = args->Outfile();
	} else if (args->fromBasename()) {
	    // set the name of the df3 output file from the basename if -b was used
	    // -b is unimplemented still... indefinately.
	    df3name = args->Basename() +  ".df3";
	} else {
	    // otherwise, if neither -o nor -b were specified, 
	    // split the first filename of incomming list to 
	    // determine what we should name the DF3 file. 
	    std::string::size_type index = file_q.at(0).find_last_of(".");
	    df3name = file_q.at(0).substr(0, index) + ".df3";
	}

	df3::DF3 df3file(df3name);
	df3file.setDepth(file_q.size()); // df3 depth is determined by how many tga files specified
	
	if (args->Verbose()) { 
	    std::cout << "tga2df3 version " << major << "." << minor << "\n";
	}
	
	// typedef a (v)ector of (s)trings iterator 
	typedef std::vector<std::string>::iterator vs_iterator; 

	/* process the ordered list of tga files */
	for (vs_iterator current = file_q.begin(); current != file_q.end(); ++current) {
	    df3file.tga2df3(*current, args);
	}
	
	std::cout << "Done. Created " << df3file.getName() << "\n";
	delete args;
	return 0;
    }
    catch (Exceptions::DF3Exceptions &e) {
	e.debug();
	if (args != NULL) { delete args; }
	return 1;
    } 
    catch (...) {
	std::cout << "\nERROR:\tUnknown error. "
		  << "Please try to reproduce the error and submit a bug report :)\n" 
		  << "tga2df3 version " << major << "." << minor << "\n";
	return 1;
    }
}
